@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-# $Id: graphdefanglib.pl,v 1.1 2003/01/22 14:36:23 dfs Exp $
+# $Id: graphdefanglib.pl,v 1.1 2003/10/22 20:08:57 dfs Exp $
 #
 # GraphDefang -- a set of tools to create graphs of your mimedefang
 #                spam and virus logs.
@@ -7,7 +7,7 @@
 # Written by:    John Kirkland
 #                jpk@bl.org
 #
-# Copyright (c) 2002, John Kirkland
+# Copyright (c) 2002-2003, John Kirkland
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 
 use strict;
 use Time::Local;
+use Time::Zone;
 use Date::Parse;
 use Date::Format;
 use Data::Dumper;
@@ -74,20 +75,16 @@ sub trim_database() {
 	my %data_db = ();
 	my %data = ();
 
-	tie (%data_db, 'MLDBM', $SUMMARYDB, O_RDWR|O_CREAT, 0644)
-		or die "Can't open $SUMMARYDB:$!\n";
-	%data = %data_db;
-
-	untie %data_db;
+	read_summarydb(\%data, O_RDONLY);
 
 	# Start the DB Trim
 
 	my $now = time();
 	my $trimcounter = 0;
 
-	# Delete hourly data older than 2*$NUM_HOURS_SUMMARY hours
+	# Delete hourly data older than 1.25*$NUM_HOURS_SUMMARY hours
 
-	my $deletetime = get_unixtime_by_timesummary('hourly',$now - 2*$NUM_HOURS_SUMMARY*60*60);
+	my $deletetime = get_unixtime_by_timesummary('hourly',$now - 1.25*$NUM_HOURS_SUMMARY*60*60);
 
 	foreach my $entrytime (keys %{$data{'hourly'}}) {
 		if ($entrytime < $deletetime) {
@@ -98,9 +95,9 @@ sub trim_database() {
 
 	print STDERR "\tTrimmed $trimcounter 'hourly' entries from SummaryDB\n" if (!$QUIET);
 
-	# Delete daily data older than 2*$NUM_DAYS_SUMMARY days
+	# Delete daily data older than 1.25*$NUM_DAYS_SUMMARY days
 
-	$deletetime = get_unixtime_by_timesummary('daily',$now - 2*$NUM_DAYS_SUMMARY*60*60*24);
+	$deletetime = get_unixtime_by_timesummary('daily',$now - 1.25*$NUM_DAYS_SUMMARY*60*60*24);
 	$trimcounter=0;
 
 	foreach my $entrytime (keys %{$data{'daily'}}) {
@@ -115,7 +112,7 @@ sub trim_database() {
 	# Delete all but Top25 entries in hours, days, and months
 	# other than the current one!
 	
-	my @DeleteTimes = ('hourly', 'daily'. 'monthly');
+	my @DeleteTimes = ('hourly', 'daily', 'monthly');
 	$trimcounter = 0;
 
 	foreach my $deletetime (@DeleteTimes) {
@@ -151,8 +148,26 @@ sub trim_database() {
 		}
 	}
 
-	print STDERR "\tTrimmed $trimcounter 'non top25' entries from SummaryDB\n" if (!$QUIET);
-							
+	print STDERR "\tTrimmed $trimcounter 'non top25' entries from SummaryDB\n" if (!$QUIET);	
+
+	backup_and_save_summarydb(\%data);
+}
+
+sub read_summarydb($$) {
+
+	my $data = shift;
+	my $method = shift;
+	my %data_db; 
+
+	tie (%data_db, 'MLDBM', $SUMMARYDB, $method, 0644)
+		or die "Can't open $SUMMARYDB:$!\n";
+	%$data = %data_db;
+	untie %data_db;
+}
+
+sub backup_and_save_summarydb($) {
+	my $dataPtr = shift;
+	my %data_db;
 
 	# Backup the old summarydb file and recreate it using the new data
 
@@ -162,71 +177,78 @@ sub trim_database() {
 	tie (%data_db, 'MLDBM', $SUMMARYDB, O_RDWR|O_CREAT, 0644)
 		or die "Can't open $SUMMARYDB:$!\n";
 
-	%data_db = %data;
+	%data_db = %$dataPtr;
 
-	untie %data_db 
+	untie %data_db
 		or die "Database Save Failed... restore from $SUMMARYDB.bak:$!\n";
+# Leave the .bak around in case the original got corrupted.
+#	unlink ("$SUMMARYDB.bak")
+#		or die "Can't unlink $SUMMARYDB.bak:$!\n";
 
-	unlink ("$SUMMARYDB.bak")
-		or die "Can't unlink $SUMMARYDB.bak:$!\n";
+	return 1;
 }
 
-sub read_and_summarize_data($) {
+sub read_and_summarize_data($$) {
+	use vars qw(%event $text $pid %spamd %user_unknown $event $value1 $value2 $sender $recipient $subject $NumEvents $FoundNewRow $unixtime $MaxDBUnixTime);
         my $fn = shift;
+	my $nomax = shift;
         my %data = ();
 	my %data_db = ();
 
-	# Temporary variables for lookup information	
-	my %sendmail = ();
-	my %spamd = ();
+	# Temporary variable for lookup information	
+	%spamd = ();
 
-	my $NumNewLines = 0;;
+	my %NumNewLines;
 	
 	# Set graphtimes
 	my @GraphTimes = ("hourly","daily","monthly");
 
-	# Open SummaryDB
-	if (!$NODB) {
-		tie (%data_db, 'MLDBM', $SUMMARYDB, O_RDWR|O_CREAT, 0644)
-			or die "Can't open $SUMMARYDB:$!\n";
-		%data = %data_db;    
+	# Load event processing perl code from the events subdirectory
+	my $dirname = "$MYDIR/event";
+	opendir(DIR, $dirname) or die "can't opendir $dirname: $!";
+	while (defined(my $file = readdir(DIR))) {
+		if (!($file =~ m/^\./) and !($file =~ m/^CVS/)) {
+			# do nothing if file starts with '.'
+			opendir(SUBDIR, "$dirname/$file") or die "can't opendir $dirname/$file: $!";
+			while (defined(my $file2 = readdir(SUBDIR))) {
+				if (!($file2 =~ m/^\./) and !($file2 =~ m/^CVS/)) {
+					require "$dirname/$file/$file2";
+				}
+			}
+		}
 	}
+	closedir(SUBDIR);
+	closedir(DIR);
+
+	# Open SummaryDB
+	read_summarydb(\%data, O_RDONLY|O_CREAT) if (!$NODB);
        
 	# Open log file 
 	tie *ZZZ, 'File::ReadBackwards', $fn || die("can't open datafile: $!");
 
 	# Get max unixtime value from DBM file 
-	my $MaxDBUnixTime = 0;
-	$MaxDBUnixTime = $data{'max'} if (defined($data{'max'}));
+	# This is left here for backwards compatibility... we now track MAX times per host 
+	# to support log files from multiples hosts.
+	$MaxDBUnixTime = 0;
+	if (!$nomax && defined($data{'max'})) {
+		$MaxDBUnixTime = $data{'max'};
+		# delete the max entry 'cuz we won't use it again
+		delete($data{'max'});
+		print STDERR "\tConverting to host-based max times\n" if (!$QUIET and !$NODB);
+		print STDERR "\tPrevious Max Unixtime from SummaryDB:  $MaxDBUnixTime\n" if (!$QUIET and !$NODB);
+	} 
 
-	print STDERR "\tMax Unixtime from SummaryDB:  $MaxDBUnixTime\n" if (!$QUIET and !$NODB);
+	# print out the list of max times per host
+	my %ReadMaxHostTime;
+	if (defined($data{'maxhosttime'})) {
+		foreach my $host (sort keys %{$data{'maxhosttime'}}) {
+			$ReadMaxHostTime{$host} = $data{'maxhosttime'}{$host};
+			print STDERR "\tMax Unixtime from SummaryDB for $host: $data{'maxhosttime'}{$host}\n" if (!$QUIET and !$NODB);
+		}
+	} 
 
-# Sample Rows from mimedefang's md_log()
+        while (<ZZZ>) {
 
-#Sep 28 21:55:50 westover mimedefang.pl[16803]: MDLOG,g8T2th86016917,mail_out,,,<mimedefang@westover.org>,<defang-admin@westover.org>,SPAM: 21.9, Make Money 100% RISK FREE!
-#Sep 28 21:55:52 westover mimedefang.pl[16803]: MDLOG,g8T2th88016917,mail_out,,,<mimedefang@westover.org>,<defang-admin@westover.org>,SPAM: 20.3, Access Your PC from Anywhere - Download Now
-#Sep 28 21:55:55 westover mimedefang.pl[16803]: MDLOG,g8T2th8A016917,mail_out,,,<mimedefang@westover.org>,<defang-admin@westover.org>,SPAM: 32.8, /ADV / The Best Business Opportunity on Net
-#Sep 28 21:55:57 westover mimedefang.pl[16803]: MDLOG,g8T2th8C016917,mail_out,,,<mimedefang@westover.org>,<defang-admin@westover.org>,SPAM: 16.6, Get the lowest long distance rates available.
-#Sep 28 21:56:02 westover mimedefang.pl[16803]: MDLOG,g8T2tt82016932,spam,38,203.167.97.19,<joisie69@hotmail.com>,<vernonm@westover.org>,I did not belive it....
-#Sep 28 21:56:09 westover mimedefang.pl[16803]: MDLOG,g8T2u782016945,mail_out,,,<mimedefang@westover.org>,<defang-admin@westover.org>,SPAM: 38, I did not belive it....
-
-# Sample Rows from spamd's syslog:
-
-# SPAM:
-#Oct  6 06:25:04 wbs sendmail[23124]: g96BP4C23124: from=<promo@spaceplanet.it>, size=1358, class=0, nrcpts=1, msgid=<200210051058.TZK5253@smtp.fastweb.it>, proto=ESMTP, daemon=MTA, relay=localhost.localdomain [127.0.0.1]
-#Oct  6 06:25:04 wbs spamd[11318]: connection from localhost.localdomain [127.0.0.1] at port 38273
-#Oct  6 06:25:04 wbs spamd[23129]: info: setuid to tex succeeded
-#Oct  6 06:25:04 wbs spamd[23129]: processing message <200210051058.TZK5253@smtp.fastweb.it> for tex:502, expecting 1623 bytes.
-#Oct  6 06:25:06 wbs spamd[23129]: identified spam (6.2/5.0) for tex:502 in 1.3 seconds, 1623 bytes.
-#CLEAN MESSAGE
-#Oct  6 06:40:28 wbs sendmail[23163]: g96BeSC23163: from=<root@moses.wbschool.net>, size=892, class=0, nrcpts=1, msgid=<200210061137.g96Bbv410245@moses.wbschool.net>, proto=ESMTP, daemon=MTA, relay=localhost.localdomain [127.0.0.1]
-#Oct  6 06:40:28 wbs spamd[11318]: connection from localhost.localdomain [127.0.0.1] at port 38318
-#Oct  6 06:40:28 wbs spamd[23168]: info: setuid to dottie succeeded
-#Oct  6 06:40:28 wbs spamd[23168]: processing message <200210061137.g96Bbv410245@moses.wbschool.net> for dottie:509, expecting 1167 bytes.
-#Oct  6 06:40:30 wbs spamd[23168]: clean message (-96.5/8.0) for dottie:509 in 1.5 seconds, 1167 bytes.
-
-        while (<ZZZ>)
-        {
                 chomp;
 
 		# Parse syslog line
@@ -241,189 +263,102 @@ sub read_and_summarize_data($) {
 		my $datestring = $1;
 		my $host = $2;
 		my $program = $3;
-		my $pid = $4;
-		my $text = $5;
-
-		my $unixtime=str2time($datestring);
+		$pid = $4;
+		$text = $5;
+	
+		# Parse date string from syslog using any TIMEZONE info from the config file.	
+		if (defined $TZ{$host}) {
+			my $zone = tz2zone($TZ{$host});
+			$unixtime=str2time($datestring,$zone);
+		} else {
+			$unixtime=str2time($datestring);
+		}
 
 		# don't examine the line if it is greater than 5 minutes
 		# older than the maximum time in our DB.  The 5 minutes
 		# comes from the PID, From, and Relay caching with sendmail
 		# and spamd that occurs below.
+		$MaxDBUnixTime = $ReadMaxHostTime{$host} if (!$nomax && defined($ReadMaxHostTime{$host}));
 		last if ($unixtime < ($MaxDBUnixTime-60*5));
 
-		my $event = '';
-		my $value1 = '';
-		my $value2 = '';
-		my $sender = '';
-		my $recipient = '';
-		my $subject = '';
+		$event = '';
+		$value1 = '';
+		$value2 = '';
+		$sender = '';
+		$recipient = '';
+		$subject = '';
 
-		my $FoundNewRow = 0;
+		$NumEvents = 1;
+		$FoundNewRow = 0;
 
-		if ($program eq 'mimedefang.pl') {
-
-			if ($text =~ m/^MDLOG,\S+?,(\S+?),(\S*?),(\S*?),(.*?),(.*?),(.*)$/ ) {
-
-				# get values from regular expression
-
-				# Only summarize data if it is newer than our current MaxDBUnixTime
-				if ($unixtime > $MaxDBUnixTime) {
-
-	              			$event = $1;
-					$value1 = $2;
-					$value2 = $3;
-					$sender = $4;
-					$recipient = $5;
-					$subject = $6;
-
-					$FoundNewRow = 1;
-				}
+		if (defined $event{$program}) {
+			foreach my $subroutine (sort keys %{$event{$program}} ) {
+				$event{$program}{$subroutine}->();
+				last if ($FoundNewRow);
 			}
-
-		} elsif ($program eq 'sendmail') {
-
-			if ($text =~ m/^\S+: from=(.+), size=.+ msgid=(.+), proto=.+ relay=(.*)$/) {
-
-                        	# our spamd times may be separated from our sendmail times by a processing
-                        	# delay.  I've set the max potential processing delay to 5 minutes here
-                        	if ($unixtime > ($MaxDBUnixTime-60*5)) {
-                                	my $from = $1;
-                                	my $msgid = $2;
-                                	my $relay = $3;
-					
-					if (defined $spamd{$msgid}) {
-						my $spamdpid = $spamd{$msgid};
-
-						$event = $spamd{$spamdpid}{'event'};
-						$sender = $from;
-						$recipient = $spamd{$spamdpid}{'for'};
-						$value1 = $spamd{$spamdpid}{'score'};
-						$value2 = $relay;
-
-						$FoundNewRow = 1;
-                                        }
-
-        			}
-			} elsif ($text =~ m/^.* (\<\S+\>)\.\.\. User unknown$/) {
-			# matching against: gAHC2iEA006157: <laches@moi.net>... User unknown
-
-				if ($unixtime > $MaxDBUnixTime) {
-					$event = 'user_unknown';
-					$recipient = $1;
-					
-					# extract the domain from the unknown user's email address
-					$recipient =~ m/<.*@(.*)>/;
-					my $domain = $1;
-					$value1 = "none";
-					$value1 = $domain if defined($domain);
-
-					$FoundNewRow = 1;
-				}
-			} elsif ($text =~ m/^.*ruleset=check_(?:rcpt|relay), .* relay=.*?(\d+\.\d+\.\d+\.\d+).* reject=(\d+) (\d\.\d.\d) .*$/) {
-			#gAABdPd17840: ruleset=check_rcpt, arg1=<0206241317.54101.bj@bl.org>, relay=200-207-163-140.dsl.telesp.net.br [200.207.163.140], reject=550 5.0.0 <0206241317.54101.bj@bl.org>... We dont accept mail from br
-			#gAF0bmd11725: ruleset=check_rcpt, arg1=<mparson@bl.org>, relay=[209.163.156.31], reject=451 4.1.8 Domain of sender address pqwa625@100pesos.com does not resolve
-				
-				if ($unixtime > $MaxDBUnixTime) {
-					$event = "reject";
-					$value1 = "$2 $3"; #reject code
-					$value2 = $1; # relay addr
-
-					$FoundNewRow = 1;
-				}
-				
-			}
-		} elsif ($program eq 'spamd') {
-		
-			if ($text =~ m/^processing message (.+) for (\S+):\d+, .+$/) {
-				
-				if ($unixtime > $MaxDBUnixTime) {
-					my $msgid = $1;
-					my $for = $2;
-
-					$spamd{$pid}{'msgid'} = $msgid;
-					$spamd{$pid}{'for'} = $for;	
-					#provide a lookup for the sendmail parsing section
-					$spamd{$msgid} = $pid;
-				}
-			} elsif ($text =~ m/^(\w+ \w+) \((.+)\/.+\) for .+$/) {
-
-				if ($unixtime > $MaxDBUnixTime) {
-					my $words = $1;
-					my $score = $2;
-
-					if ($words eq 'identified spam') { $event = 'spam' }
-					elsif ($words eq 'clean message') { $event = 'non-spam' }
-
-					$spamd{$pid}{'event'} = $event;
-					$spamd{$pid}{'score'} = $score;
-				}
-			}
-		} elsif ($program eq 'ipop3d') {
-# Searching for Oct 17 21:58:33 westover ipop3d[21207]: Login user=ben host=c68.115.81.145.roc.mn.charter.com [68.115.81.145] nmsgs=0/0
-			if ($text =~ m/^Login user=(\S+) host=(\S+)/) {
-				
-				if ($unixtime > $MaxDBUnixTime) {
-					my $user = $1;
-					my $host = $2;
-
-					$event = 'ipop3d';
-					$value1 = $user;
-					$value2 = $host;
-
-					$FoundNewRow = 1;
-				}
-			}
-		} elsif ($program eq 'kernel') {
-#Oct 27 04:02:30 hdnetwork kernel: gShield (default drop) IN=eth0 OUT= MAC=ff:ff:ff:ff:ff:ff:00:a0:c8:08:2b:2c:08:00 SRC=66.139.79.158 DST=216.201.156.32 LEN=60 TOS=0x00 PREC=0x00 TTL=52 ID=49735 DF PROTO=TCP SPT=51023 DPT=21 WINDOW=5840 RES=0x00 SYN URGP=0
-                        if ($text =~ m/^gShield .* SRC=(\S+) DST=(\S+) .* DPT=(\d+)/) {
-                                if ($unixtime > $MaxDBUnixTime) {
-                                        my $src = $1;
-                                        my $dst = $2;
-                                        my $dpt = $3;
-
-                                        $event =  'gShield';
-                                        $sender = $src;
-                                        $recipient = $dst;
-                                        $value1 = $dpt;
-
-                                        $FoundNewRow = 1;
-                                }
-                        }
-                }
+		}
 
 		if ($FoundNewRow) {
 			# Increment Number of New Lines Found
-			$NumNewLines++;
-			
+			$NumNewLines{$host}++;
+
 			# rollup hourly, daily, and monthly summaries for every variable
 			foreach my $timesummary (@GraphTimes) {
 
 				my $summarytime = get_unixtime_by_timesummary($timesummary, $unixtime);
-				$data{$timesummary}{$summarytime}{$event}{'summary'}++;
-				$data{$timesummary}{$summarytime}{$event}{'value1'}{$value1}++ 		if ($value1 ne '');
-				$data{$timesummary}{$summarytime}{$event}{'value2'}{$value2}++ 		if ($value2 ne '');
-				$data{$timesummary}{$summarytime}{$event}{'sender'}{$sender}++ 		if ($sender ne '');;
-				$data{$timesummary}{$summarytime}{$event}{'recipient'}{$recipient}++ 	if ($recipient ne '');
-				$data{$timesummary}{$summarytime}{$event}{'subject'}{$subject}++     	if ($subject ne '');
+				$data{$timesummary}{$summarytime}{$event}{'summary'}+=$NumEvents;
+				$data{$timesummary}{$summarytime}{$event}{'value1'}{$value1}+=$NumEvents 	if ($value1 ne '');
+				$data{$timesummary}{$summarytime}{$event}{'value2'}{$value2}+=$NumEvents	if ($value2 ne '');
+				$data{$timesummary}{$summarytime}{$event}{'sender'}{$sender}+=$NumEvents	if ($sender ne '');;
+				$data{$timesummary}{$summarytime}{$event}{'recipient'}{$recipient}+=$NumEvents 	if ($recipient ne '');
+				$data{$timesummary}{$summarytime}{$event}{'subject'}{$subject}+=$NumEvents     	if ($subject ne '');
 
 				# Store the maximum unixtime per timesummary for later reference
-				$data{'max'} = $unixtime 
-					if (!defined($data{'max'}) 
-						or $unixtime > $data{'max'});
+				$data{'maxhosttime'}{$host} = $unixtime 
+					if (!defined($data{'maxhosttime'}{$host}) 
+						or $unixtime > $data{'maxhosttime'}{$host});
 			} 
 		}
 	}
         close (ZZZ);
 
 	if (!$NODB) {
-		%data_db = %data;
-		untie %data_db;
-		print STDERR "\t$NumNewLines new log lines loaded\n" if (!$QUIET);
+		if (backup_and_save_summarydb(\%data)) {
+			if (%NumNewLines) {
+				foreach my $host (sort keys %NumNewLines) {
+					print STDERR "\t$NumNewLines{$host} new log lines processed for $host\n" if (!$QUIET);
+				}
+			} else {
+				print STDERR "\t 0 new log lines processed\n" if (!$QUIET);
+			}
+		}
 	}
         return %data;
 }
 
+sub get_all_data_types($) {
+
+	my $dataPtr = shift;
+
+	my %all;
+	my @return_all;
+
+	# get list of potential event values
+
+	foreach my $date (keys %{$dataPtr->{monthly}}) {
+		foreach my $data_type (keys %{$dataPtr->{monthly}{$date}}) {
+			$all{$data_type} = 1;
+		}
+	}
+
+	foreach my $key (sort keys %all) {
+		push @return_all, $key;
+	}
+
+
+	return @return_all;
+}
+
 sub graph($$) {
 	my $settings = shift;
 	my $data = shift;
@@ -436,7 +371,7 @@ sub graph($$) {
 		# Set the settings for the graph we've been asked to draw
 		set_graph_settings($settings);
 
-		print STDERR "\t$settings->{filename}\n" if (!$QUIET);
+		print STDERR "\t$settings->{chart_filename}\n" if (!$QUIET);
 
 		# Get the data for the graph we've been asked to draw
 		my @GraphData = get_graph_data($settings,$data);
@@ -460,9 +395,15 @@ sub set_graph_settings($) {
 
         # Initialize the title and filename
         $settings->{chart_title} = "";
-        $settings->{filename} = "";
+        $settings->{chart_filename} = "";
 
 	my $autotitle = "";
+	my $autofilename = "";
+
+	# Set graph x & y dimensions
+
+	$settings->{x_graph_size} = $X_GRAPH_SIZE if (!defined($settings->{x_graph_size}));
+	$settings->{y_graph_size} = $Y_GRAPH_SIZE if (!defined($settings->{y_graph_size}));
 
         # Add "Top N" to the beginning of the Title if necessary
         if ($settings->{top_n}) {
@@ -474,7 +415,7 @@ sub set_graph_settings($) {
 	foreach my $data_type (@{$settings->{data_types}}) {
 		# Uppercase the first letter of the data_type
 		$autotitle .= "\u$data_type";
-		$settings->{filename} .= $data_type;
+		$autofilename .= $data_type;
 		$i++;
 		if ( $i == ($#{$settings->{data_types}}) ) {
 			$autotitle .= " and "
@@ -511,57 +452,66 @@ sub set_graph_settings($) {
 	# Put top_n in the filename?
 
 	if ($settings->{top_n}) {
-        	$settings->{filename} .= "_$settings->{top_n}";
+        	$autofilename .= "_$settings->{top_n}";
 	} else {
-		$settings->{filename} .= "_";
+		$autofilename .= "_";
 	}
 
-	$settings->{filename} .= "$settings->{grouping}_$settings->{graph_type}";
+	$autofilename .= "$settings->{grouping}_$settings->{graph_type}";
 
         # The final portion of the title will be set in the section below
 
         if ($settings->{grouping_time} eq 'hourly') {
 
                 $settings->{x_axis_num_values}  = $NUM_HOURS_SUMMARY;   # Number of x-axis values on graph
+		$settings->{x_axis_num_values}  = $settings->{num_hourly_values} if defined($settings->{num_hourly_values});
                 $settings->{x_axis_num_sec_incr}= 60*60;                # Incremental number of seconds represented by each x-axis value
                 $settings->{x_axis_date_format} = "%h %d, %I%p";        # Format of date string on x-axis
                 $settings->{x_label}            = 'Hours';
                 $settings->{y_label}            = 'Counts per Hour';
                 $settings->{chart_title}        = $autotitle . " per Hour (last $settings->{x_axis_num_values} hours)"
                                                 unless defined($settings->{title});
-                $settings->{filename}           = "hourly_" . $settings->{filename};
+                $autofilename			= "hourly_" . $autofilename;
 
         } elsif ($settings->{grouping_time} eq 'daily') {
 
                 $settings->{x_axis_num_values}  = $NUM_DAYS_SUMMARY;
+		$settings->{x_axis_num_values}  = $settings->{num_daily_values} if defined($settings->{num_daily_values});
                 $settings->{x_axis_num_sec_incr}= 60*60*24;
                 $settings->{x_axis_date_format} = "%h %d";
                 $settings->{x_label}            = 'Days';
                 $settings->{y_label}            = 'Counts per Day';
                 $settings->{chart_title}              = $autotitle . " per Day (Last $settings->{x_axis_num_values} days)"
                                                 unless defined($settings->{title});
-                $settings->{filename}      = "daily_" . $settings->{filename};
+                $autofilename      = "daily_" . $autofilename;
 
         } elsif ($settings->{grouping_time} eq 'monthly') {
 
                 $settings->{x_axis_num_values}  = $NUM_MONTH_SUMMARY;
+		$settings->{x_axis_num_values}  = $settings->{num_monthly_values} if defined($settings->{num_monthly_values});
                 $settings->{x_axis_num_sec_incr}= 60*60*24*31;
                 $settings->{x_axis_date_format} = "%h";
                 $settings->{x_label}            = 'Months';
                 $settings->{y_label}            = 'Counts per Month';
                 $settings->{chart_title}              = $autotitle . " per Month (Last $settings->{x_axis_num_values} months)"
                                                 unless defined($settings->{title});
-                $settings->{filename}           = "monthly_" . $settings->{filename};
+                $autofilename           = "monthly_" . $autofilename;
         }
 
 	if (defined $settings->{filter_name}) {
 		my $filter;
 		($filter = $settings->{filter_name}) =~ s/\W/_/g;
 		$settings->{chart_title} .= " filtered by $settings->{filter_name}";
-		$settings->{filename} .= "_$filter";
+		$autofilename .= "_$filter";
 	}
 
+	# Use the title from graphdefang-config if specified, else use the autotitle
 	$settings->{chart_title} = $settings->{title} if (defined($settings->{title}));
+
+	# Use the filename from graphdefang-config if specified, else use the autofilename
+	$settings->{chart_filename} = $autofilename;
+	$settings->{chart_filename} = "$settings->{grouping_time}_$settings->{filename}" if (defined($settings->{filename}));
+	$settings->{chart_filename} =~ s/\//_/g; # Replace any '/' chars with '_'
 }
 
 
@@ -574,9 +524,15 @@ sub get_graph_data($$) {
 
         my $currenttime = time();
 	my $cutofftime;
+	my $currentyear;
+	my $currentmon;
+	my $currentisdst;
 
 	if ($settings->{grouping_time} eq 'monthly') {
 		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($currenttime);
+		$currentyear = $year;
+		$currentmon = $mon;
+		$currentisdst = $isdst;
                 # Decrement the month/year value n times
 		for (my $i = 0; $i < $settings->{x_axis_num_values}-1; $i++) {
                        	if ($mon == 0) {
@@ -588,6 +544,7 @@ sub get_graph_data($$) {
                 }
 
                 # get unixtime for our new values
+		$mday = 1; # Get around a bug that only shows itself on the 30th or 31st of the month
                 $cutofftime = timelocal($sec, $min, $hour, $mday, $mon, $year);
 
 	} else {
@@ -616,10 +573,8 @@ sub get_graph_data($$) {
 			push @{$settings->{'data_types'}}, $key;
 		}
 	}
-
 	# Summarize totals across time interval
 	for (my $time=$cutofftime; $time<=$currenttime; $time += $settings->{x_axis_num_sec_incr}) {
-
 		my $date = get_unixtime_by_timesummary($settings->{grouping_time},$time);
 
 		# Get total for summary grouping
@@ -673,10 +628,22 @@ sub get_graph_data($$) {
 				$mon++;
 			}
 
+			# Has dst kicked in this month?  If so, adjust for it
+			my $dstadjustment = 0;
+			if (($currentyear == $year) && ($currentmon == $mon)) {
+				if ($currentisdst > $isdst) {
+					$dstadjustment = -3600;
+				} elsif ($currentisdst < $isdst) {
+					$dstadjustment = 3600;
+				} else {
+					$dstadjustment = 0;
+				}
+			}
+			
 			# get unixtime for our new values
 			my $newmonthtime = timelocal($sec, $min, $hour, $mday, $mon, $year);
 
-			$settings->{x_axis_num_sec_incr} = $newmonthtime - $date;
+			$settings->{x_axis_num_sec_incr} = $newmonthtime - $date + $dstadjustment;
                 }
 	}
 
@@ -706,7 +673,14 @@ sub get_graph_data($$) {
 	for (my $time=$cutofftime; $time<=$currenttime; $time += $settings->{x_axis_num_sec_incr}) {
 
 		my $date = get_unixtime_by_timesummary($settings->{grouping_time},$time);
-		my $datestring = time2str($settings->{x_axis_date_format},$date);
+
+		my $datestring = '';
+		if (defined($TZ{GD_Display})) {
+			my $zone = tz2zone($TZ{GD_Display});
+			$datestring = time2str($settings->{x_axis_date_format},$date,$zone);
+		} else {
+			$datestring = time2str($settings->{x_axis_date_format},$date);
+		}
 
 		my $i=0;
 		push @{$GraphData[$i]}, $datestring;
@@ -765,10 +739,22 @@ sub get_graph_data($$) {
 				$mon++;
 			}
 
+			# Has dst kicked in this month?  If so, adjust for it
+			my $dstadjustment = 0;
+			if (($currentyear == $year) && ($currentmon == $mon)) {
+				if ($currentisdst > $isdst) {
+					$dstadjustment = -3600;
+				} elsif ($currentisdst < $isdst) {
+					$dstadjustment = 3600;
+				} else {
+					$dstadjustment = 0;
+				}
+			}
+
 			# get unixtime for our new values
 			my $newmonthtime = timelocal($sec, $min, $hour, $mday, $mon, $year);
 
-			$settings->{x_axis_num_sec_incr} = $newmonthtime - $date;
+			$settings->{x_axis_num_sec_incr} = $newmonthtime - $date + $dstadjustment;
                 }
 	}	
 
@@ -780,8 +766,7 @@ sub get_graph_data($$) {
 sub draw_line_graph($$) {
         my $settings = shift;
         my $data = shift;
-
-        my $my_graph = new GD::Graph::linespoints($X_GRAPH_SIZE, $Y_GRAPH_SIZE);
+        my $my_graph = new GD::Graph::linespoints($settings->{x_graph_size}, $settings->{y_graph_size});
 
         $my_graph->set(
                 x_label                 => $settings->{x_label},
@@ -808,14 +793,14 @@ sub draw_line_graph($$) {
 
         $my_graph->set_legend( @{$settings->{legend}} );
         $my_graph->plot($data);
-        save_chart($my_graph, "$OUTPUT_DIR/$settings->{filename}");
+        save_chart($my_graph, "$OUTPUT_DIR/$settings->{chart_filename}");
 }
 
 sub draw_stacked_bar_graph($$) {
         my $settings = shift;
         my $data = shift;
 
-        my $my_graph = new GD::Graph::bars($X_GRAPH_SIZE, $Y_GRAPH_SIZE);
+        my $my_graph = new GD::Graph::bars($settings->{x_graph_size}, $settings->{y_graph_size});
 
         $my_graph->set(
                 x_label                 => $settings->{x_label},
@@ -840,7 +825,7 @@ sub draw_stacked_bar_graph($$) {
 
         $my_graph->set_legend( @{$settings->{legend}} );
         $my_graph->plot($data);
-        save_chart($my_graph, "$OUTPUT_DIR/$settings->{filename}");
+        save_chart($my_graph, "$OUTPUT_DIR/$settings->{chart_filename}");
 }
 
 sub save_chart($$) {
