@@ -9,7 +9,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: embperl.c,v 1.11 2004/10/29 13:33:25 dfs Exp $";
+"$Id: embperl.c,v 1.12 2004/11/25 13:40:40 dfs Exp $";
 
 #ifdef EMBED_PERL
 #include <EXTERN.h>
@@ -99,7 +99,7 @@ make_embedded_interpreter(char const *progPath,
     perl_parse(my_perl, xs_init, argc, argv, NULL);
     perl_run(my_perl);
     if (num_open_descriptors() > descriptors) {
-	syslog(LOG_CRIT, "WARNING: Something in your Perl filter appears to have opened a file descriptor outside of any function.  With embedded Perl, you should move any code that opens a file descriptor into filter_initialize.  DON'T BLAME MIMEDEFANG IF YOUR FILTER FAILS IN MYSTERIOUS AND UNPREDICTABLE WAYS.");
+	syslog(LOG_INFO, "WARNING: Something in your Perl filter appears to have opened a file descriptor outside of any function.  With embedded Perl, you should move any code that opens a file descriptor into filter_initialize.  On some systems, the C library may open a descriptor, but you should verify your filter just in case.");
     }
     return 0;
 }
