@@ -5,7 +5,7 @@
 * C interface to the attachment-filter program for stripping or altering
 * MIME attachments in incoming Sendmail connections.
 *
-* Copyright (C) 2000 Roaring Penguin Software Inc.
+* Copyright (C) 2000-2005 Roaring Penguin Software Inc.
 * http://www.roaringpenguin.com
 *
 * This program may be distributed under the terms of the GNU General
@@ -16,7 +16,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: mimedefang.c,v 1.216 2004/10/27 13:09:43 dfs Exp $";
+"$Id: mimedefang.c,v 1.224 2005/02/08 17:56:31 dfs Exp $";
 
 /* Define this to work around an M$ Outlook bug! */
 /* #define CONVERT_EMBEDDED_CRS_IN_HEADERS 1 */
@@ -79,6 +79,10 @@ typedef int md_socklen_t;
 #define DEBUG(x) (void) 0
 #endif
 
+/* If we don't have inet_ntop, we need to protect inet_ntoa with a mutex */
+#ifndef HAVE_INET_NTOP
+static pthread_mutex_t ntoa_mutex = PTHREAD_MUTEX_INITIALIZER;
+#endif
 extern int find_syslog_facility(char const *facility_name);
 
 #define DEBUG_ENTER(func, line) DEBUG(syslog(LOG_DEBUG, "Entering %s (line %d)", func, line))
@@ -106,6 +110,9 @@ int LogTimes = 0;
 /* Default Backlog for "listen" */
 static int Backlog = -1;
 
+/* Allow new connections to queue? */
+static int AllowNewConnectionsToQueue = 0;
+
 /* Run as this user */
 static char *user = NULL;
 extern int drop_privs(char const *user, uid_t uid, gid_t gid);
@@ -114,7 +121,10 @@ extern int drop_privs(char const *user, uid_t uid, gid_t gid);
 static char *NOQUEUE = "NOQUEUE";
 
 /* My IP address */
-static char *MyIPAddress;
+static char *MyIPAddress = NULL;
+
+/* "Equivalent-to-loopback" address */
+static char *EquivToLoopback = NULL;
 
 /* Header name for validating IP addresses */
 static char ValidateHeader[256];
@@ -359,7 +369,6 @@ static sfsistat
 mfconnect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *sa)
 {
     struct privdata *data;
-    int n;
 
     char const *tmp;
     struct sockaddr_in *insa = (struct sockaddr_in *) sa;
@@ -373,16 +382,18 @@ mfconnect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *sa)
     mfclose(ctx);
 
     /* If too many running filters, reject connection at this phase */
-    n = MXCheckFreeSlaves(MultiplexorSocketName);
-    if (n == 0) {
-	syslog(LOG_WARNING, "mfconnect: No free slaves");
-	DEBUG_EXIT("mfconnect", __LINE__, "SMFIS_TEMPFAIL");
-	return SMFIS_TEMPFAIL;
-    }
-    if (n < 0) {
-	syslog(LOG_WARNING, "mfconnect: Error communicating with multiplexor");
-	DEBUG_EXIT("mfconnect", __LINE__, "SMFIS_TEMPFAIL");
-	return SMFIS_TEMPFAIL;
+    if (!AllowNewConnectionsToQueue) {
+	int n = MXCheckFreeSlaves(MultiplexorSocketName);
+	if (n == 0) {
+	    syslog(LOG_WARNING, "mfconnect: No free slaves");
+	    DEBUG_EXIT("mfconnect", __LINE__, "SMFIS_TEMPFAIL");
+	    return SMFIS_TEMPFAIL;
+	}
+	if (n < 0) {
+	    syslog(LOG_WARNING, "mfconnect: Error communicating with multiplexor");
+	    DEBUG_EXIT("mfconnect", __LINE__, "SMFIS_TEMPFAIL");
+	    return SMFIS_TEMPFAIL;
+	}
     }
 
     data = malloc_with_log(sizeof *data);
@@ -454,8 +465,10 @@ mfconnect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *sa)
 	    return SMFIS_TEMPFAIL;
 	}
 #else
+	pthread_mutex_lock(&ntoa_mutex);
 	tmp = inet_ntoa(insa->sin_addr);
 	if (tmp) strncpy(data->hostip, tmp, 64);
+	pthread_mutex_unlock(&ntoa_mutex);
 #endif
 	if (!tmp) {
 	    syslog(LOG_WARNING, "inet_ntoa or inet_ntop failed: %m");
@@ -466,6 +479,14 @@ mfconnect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *sa)
 	data->hostip[64] = 0;
     }
 
+    /* Is host IP equivalent to loopback?  If so, replace with 127.0.0.1 */
+    /* (We have enough room because data->hostip can hold 64 chars + nul) */
+    if (data->hostip    &&
+	EquivToLoopback &&
+	!strcmp(data->hostip, EquivToLoopback)) {
+	strcpy(data->hostip, "127.0.0.1");
+    }
+
     data->dir = NULL;
     data->fd = -1;
     data->headerFD = -1;
@@ -692,6 +713,11 @@ envfrom(SMFICTX *ctx, char **from)
     /* Write the sender */
     write_mx_command(data->cmdFD, 'S', (unsigned char *) from[0]);
 
+    /* Write ESMTP args */
+    for (i=1; from[i]; i++) {
+	write_mx_command(data->cmdFD, 's', (unsigned char *) from[i]);
+    }
+
     /* Write some macros */
     write_macro_value(ctx, "_");
     write_macro_value(ctx, "auth_authen");
@@ -720,7 +746,7 @@ envfrom(SMFICTX *ctx, char **from)
 
     /* Get my IP address */
     me = smfi_getsymval(ctx, "{if_addr}");
-    if (me && *me && !strcmp(me, "127.0.0.1")) {
+    if (me && *me && strcmp(me, "127.0.0.1")) {
 	data->myip = strdup_with_log(me);
     } else {
 	/* Sigh... use our computed address */
@@ -730,7 +756,7 @@ envfrom(SMFICTX *ctx, char **from)
 
     if (doSenderCheck) {
 	int n = MXSenderOK(MultiplexorSocketName, buf2,
-			   from[0], data->hostip, data->hostname,
+			   (char const **) from, data->hostip, data->hostname,
 			   data->heloArg, data->dir, data->qid);
 	if (n == 0) {
 	    set_dsn(ctx, buf2, 5);
@@ -798,6 +824,7 @@ rcptto(SMFICTX *ctx, char **to)
     char ans[SMALLBUF];
     sfsistat retcode = SMFIS_CONTINUE;
     char const *rcpt_mailer, *rcpt_host, *rcpt_addr;
+    int i;
 
     DEBUG_ENTER("rcptto", __LINE__);
     if (!data) {
@@ -827,7 +854,7 @@ rcptto(SMFICTX *ctx, char **to)
 	    }
 	}
 	n = MXRecipientOK(MultiplexorSocketName, ans,
-			  to[0], data->sender, data->hostip,
+			  (char const **) to, data->sender, data->hostip,
 			  data->hostname, data->firstRecip, data->heloArg,
 			  data->dir, data->qid,
 			  rcpt_mailer, rcpt_host, rcpt_addr);
@@ -877,6 +904,12 @@ rcptto(SMFICTX *ctx, char **to)
     writestr(data->cmdFD, " ");
     write_percent_encoded((unsigned char *) rcpt_addr, data->cmdFD);
     writestr(data->cmdFD, "\n");
+
+    /* Write ESMTP args */
+    for (i=1; to[i]; i++) {
+	write_mx_command(data->cmdFD, 'r', (unsigned char *) to[i]);
+    }
+
     data->cmdFD = put_fd(data->cmdFD);
     DEBUG_EXIT("rcptto", __LINE__, "SMFIS_CONTINUE or SMFIS_ACCEPT");
     return retcode;
@@ -1647,6 +1680,7 @@ usage(void)
     fprintf(stderr, "  -r                -- Do relay check before processing body (requires -m)\n");
     fprintf(stderr, "  -s                -- Do sender check before processing body (requires -m)\n");
     fprintf(stderr, "  -t                -- Do recipient checks before processing body (requires -m)\n");
+    fprintf(stderr, "  -q                -- Allow new connections to be queued by multiplexor\n");
     fprintf(stderr, "  -P file           -- Write process-ID of daemon to specified file\n");
     fprintf(stderr, "  -T                -- Log filter times to syslog\n");
     fprintf(stderr, "  -b n              -- Set listen() backlog to n\n");
@@ -1657,6 +1691,7 @@ usage(void)
     fprintf(stderr, "  -D                -- Do not become a daemon (stay in foreground)\n");
     fprintf(stderr, "  -S facility       -- Set syslog(3) facility\n");
     fprintf(stderr, "  -a macro          -- Pass additional Sendmail macro\n");
+    fprintf(stderr, "  -L ip.addr        -- Specify 'equivalent-to-loopback' address\n");
     exit(EXIT_FAILURE);
 }
 
@@ -1699,6 +1734,7 @@ main(int argc, char **argv)
     }
 
     MyIPAddress = NULL;
+    EquivToLoopback = NULL;
 
     /* Determine my IP address */
     if (gethostname(buf, sizeof(buf)) >= 0) {
@@ -1712,7 +1748,7 @@ main(int argc, char **argv)
 	    }
 #else
 	    {
-		char *s = inet_ntoa(in.sin_addr);
+		char *s = inet_ntoa(in);
 		if (s && *s) MyIPAddress = strdup_with_log(s);
 	    }
 #endif
@@ -1723,8 +1759,11 @@ main(int argc, char **argv)
     }
 
     /* Process command line options */
-    while ((c = getopt(argc, argv, "a:Chp:dm:srtkP:U:Tx:MXS:Dvb:")) != -1) {
+    while ((c = getopt(argc, argv, "a:Chp:dm:srtkP:U:Tx:MXS:Dvb:L:q")) != -1) {
 	switch (c) {
+	case 'q':
+	    AllowNewConnectionsToQueue = 1;
+	    break;
 	case 'b':
 	    sscanf(optarg, "%d", &Backlog);
 	    if (Backlog < 5) Backlog = 5;
@@ -1781,6 +1820,17 @@ main(int argc, char **argv)
 	    }
 	    break;
 
+	case 'L':
+	    if (EquivToLoopback) {
+		free(EquivToLoopback);
+	    }
+	    EquivToLoopback = strdup(optarg);
+	    if (!EquivToLoopback) {
+		fprintf(stderr, "%s: Out of memory\n", argv[0]);
+		exit(EXIT_FAILURE);
+	    }
+	    break;
+
 	case 'x':
 	    if (scan_body) {
 		free(scan_body);
@@ -2115,7 +2165,7 @@ do_sm_quarantine(SMFICTX *ctx,
 		 char const *reason)
 {
 #ifdef SMFIF_QUARANTINE
-    return smfi_quarantine(ctx, reason);
+    return smfi_quarantine(ctx, (char *) reason);
 #else
     syslog(LOG_WARNING, "smfi_quarantine not supported: Requires Sendmail 8.13.0 or later");
     return MI_FAILURE;
