@@ -4,7 +4,7 @@
 *
 * Notification routines for multiplexor
 *
-* Copyright (C) 2002-2004 Roaring Penguin Software Inc.
+* Copyright (C) 2002-2005 Roaring Penguin Software Inc.
 * http://www.roaringpenguin.com
 *
 * This program may be distributed under the terms of the GNU General
@@ -13,7 +13,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: notifier.c,v 1.10 2004/08/24 14:23:18 dfs Exp $";
+"$Id: notifier.c,v 1.12 2005/02/08 16:04:40 dfs Exp $";
 
 #include "config.h"
 #include "event_tcp.h"
@@ -67,7 +67,7 @@ make_notifier_socket(EventSelector *es, char const *name)
     int sock;
     int i;
 
-    sock = make_listening_socket(name, 5);
+    sock = make_listening_socket(name, 5, 0);
     if (sock < 0) {
 	return -1;
     }
