@@ -4,7 +4,7 @@
 *
 * Utility functions for MIMEDefang
 *
-* Copyright (C) 2002-2003 Roaring Penguin Software Inc.
+* Copyright (C) 2002-2005 Roaring Penguin Software Inc.
 * http://www.roaringpenguin.com
 *
 * This program may be distributed under the terms of the GNU General
@@ -13,7 +13,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: utils.c,v 1.42 2004/09/22 14:06:03 dfs Exp $";
+"$Id: utils.c,v 1.45 2005/02/08 16:04:40 dfs Exp $";
 
 #define _BSD_SOURCE 1
 
@@ -61,6 +61,9 @@ typedef unsigned int uint32_t;
 #define INADDR_LOOPBACK 0x7f000001
 #endif
 
+static int percent_encode_command(int term_with_newline,
+				  char *out, int outlen, ...);
+
 /**********************************************************************
 * %FUNCTION: split_on_space
 * %ARGUMENTS:
@@ -500,13 +503,14 @@ percent_encode(unsigned char *in,
 	    break;
 	}
     }
-    out[nwritten] = 0;
+    *out = 0;
     return nwritten;
 }
 
 /**********************************************************************
 * %FUNCTION: percent_encode_command
 * %ARGUMENTS:
+*  term_with_newline -- if true, terminate with "\n\0".  Otherwise, just "\0"
 *  out -- output buffer
 *  outlen -- length of output buffer
 *  args -- arguments.  Each one is percent-encoded and space-separated from
@@ -515,10 +519,9 @@ percent_encode(unsigned char *in,
 *  0 if everything fits; -1 otherwise.
 * %DESCRIPTION:
 *  Writes a series of space-separated, percent-encoded words to a buffer.
-*  Buffer is terminated with "\n\0"
 ***********************************************************************/
-int
-percent_encode_command(char *out, int outlen, ...)
+static int
+percent_encode_command(int term_with_newline, char *out, int outlen, ...)
 {
     va_list ap;
     int spaceleft = outlen-2;
@@ -546,7 +549,7 @@ percent_encode_command(char *out, int outlen, ...)
 	out += len;
     }
     va_end(ap);
-    *out++ = '\n';
+    if (term_with_newline) *out++ = '\n';
     *out = 0;
     return 0;
 }
@@ -634,7 +637,7 @@ MXRelayOK(char const *sockname,
     if (!name || !*name) {
 	name = ip;
     }
-    if (percent_encode_command(cmd, sizeof(cmd), "relayok", ip, name, NULL) < 0) {
+    if (percent_encode_command(1, cmd, sizeof(cmd), "relayok", ip, name, NULL) < 0) {
 	return -1;
     }
     if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return -1;
@@ -646,7 +649,8 @@ MXRelayOK(char const *sockname,
 * %ARGUMENTS:
 *  sockname -- socket name
 *  msg -- buffer of at least SMALLBUF size for error message
-*  sender -- sender's e-mail address
+*  sender_argv -- args from sendmail.  sender_argv[0] is sender; rest are
+*                 ESMTP args.
 *  ip -- sending relay's IP address
 *  name -- sending relay's host name
 *  helo -- argument to "HELO/EHLO" (may be NULL)
@@ -660,7 +664,7 @@ MXRelayOK(char const *sockname,
 int
 MXSenderOK(char const *sockname,
 	   char *msg,
-	   char const *sender,
+	   char const **sender_argv,
 	   char const *ip,
 	   char const *name,
 	   char const *helo,
@@ -669,6 +673,9 @@ MXSenderOK(char const *sockname,
 {
     char cmd[SMALLBUF];
     char ans[SMALLBUF];
+    int l, l2, i;
+
+    char const *sender = sender_argv[0];
 
     *msg = 0;
 
@@ -685,10 +692,31 @@ MXSenderOK(char const *sockname,
     if (!helo) {
 	helo = "UNKNOWN";
     }
-    if (percent_encode_command(cmd, sizeof(cmd), "senderok", sender, ip, name,
+    if (percent_encode_command(0, cmd, sizeof(cmd)-1, "senderok", sender, ip,
+			       name,
 			       helo, dir, qid, NULL) < 0) {
 	return -1;
     }
+
+    /* Append ESMTP args */
+    l = strlen(cmd);
+    for (i=1; sender_argv[i]; i++) {
+	percent_encode((unsigned char *) sender_argv[i],
+		       ans,
+		       sizeof(ans));
+	l2 = strlen(ans) + 1;
+	if (l + l2 < sizeof(cmd)-1) {
+	    strcat(cmd, " ");
+	    strcat(cmd, ans);
+	    l += l2;
+	} else {
+	    break;
+	}
+    }
+
+    /* Add newline */
+    strcat(cmd, "\n");
+
     if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return -1;
     return munch_mx_return(ans, msg);
 }
@@ -698,7 +726,7 @@ MXSenderOK(char const *sockname,
 * %ARGUMENTS:
 *  sockname -- multiplexor socket name
 *  msg -- buffer of at least SMALLBUF size for error messages
-*  recipient -- recipient e-mail address
+*  recip_argv -- recipient e-mail address and ESMTP args
 *  sender -- sender's e-mail address
 *  ip -- sending relay's IP address
 *  name -- sending relay's host name
@@ -716,7 +744,7 @@ MXSenderOK(char const *sockname,
 int
 MXRecipientOK(char const *sockname,
 	      char *msg,
-	      char const *recipient,
+	      char const **recip_argv,
 	      char const *sender,
 	      char const *ip,
 	      char const *name,
@@ -731,6 +759,8 @@ MXRecipientOK(char const *sockname,
 {
     char cmd[SMALLBUF];
     char ans[SMALLBUF];
+    int i, l, l2;
+    char const *recipient = recip_argv[0];
 
     *msg = 0;
 
@@ -756,12 +786,32 @@ MXRecipientOK(char const *sockname,
 	helo = "UNKNOWN";
     }
 
-    if (percent_encode_command(cmd, sizeof(cmd),
+    if (percent_encode_command(0, cmd, sizeof(cmd),
 			       "recipok", recipient, sender, ip, name, firstRecip,
 			       helo, dir, qid, rcpt_mailer, rcpt_host, rcpt_addr,
 			       NULL) < 0) {
 	return -1;
     }
+
+    /* Append ESMTP args */
+    l = strlen(cmd);
+    for (i=1; recip_argv[i]; i++) {
+	percent_encode((unsigned char *) recip_argv[i],
+		       ans,
+		       sizeof(ans));
+	l2 = strlen(ans) + 1;
+	if (l + l2 < sizeof(cmd)-1) {
+	    strcat(cmd, " ");
+	    strcat(cmd, ans);
+	    l += l2;
+	} else {
+	    break;
+	}
+    }
+
+    /* Add newline */
+    strcat(cmd, "\n");
+
     if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return -1;
     return munch_mx_return(ans, msg);
 }
@@ -980,13 +1030,15 @@ validate_smtp_dsn(char const *dsn,
 *                   inet:port  (host defaults to LOOPBACK)
 *                   inet_any:port (host defaults to INADDR_ANY)
 *  backlog -- listen backlog.  If -1, we use default of 5
+*  must_be_unix -- If true, do not accept the inet: or inet_any: sockets.
 * %RETURNS:
-*  A listening UNIX-domain or TCP socket
+*  A listening UNIX-domain or TCP socket.  If must_be_unix is true
+*  and we asked for a TCP socket, return -2.  Other failures return -1.
 * %DESCRIPTION:
 *  Utility function for opening a listening socket.
 ***********************************************************************/
 int
-make_listening_socket(char const *str, int backlog)
+make_listening_socket(char const *str, int backlog, int must_be_unix)
 {
     char const *path;
     int sock;
@@ -1000,6 +1052,9 @@ make_listening_socket(char const *str, int backlog)
 
     if (!strncmp(str, "inet:", 5) ||
 	!strncmp(str, "inet_any:", 9)) {
+	if (must_be_unix) {
+	    return -2;
+	}
 	if (!strncmp(str, "inet:", 5)) {
 	    path = str+5;
 	    bind_addr = htonl(INADDR_LOOPBACK);
